@@ -58,6 +58,7 @@ from .process import (
 from .singularity import SingularityCommandLineJob
 from .stdfsaccess import StdFsAccess
 from .udocker import UDockerCommandLineJob
+from .update import ORDERED_VERSIONS
 from .utils import (
     CWLObjectType,
     CWLOutputType,
@@ -79,7 +80,6 @@ from .utils import (
     visit_class,
     windows_default_container_id,
 )
-from .update import ORDERED_VERSIONS
 
 if TYPE_CHECKING:
     from .provenance_profile import ProvenanceProfile  # pylint: disable=unused-import
@@ -485,7 +485,8 @@ class CommandLineTool(Process):
             # "listing" is an array of either expressions or Dirent so
             # evaluate each item
             for t in cast(
-                MutableSequence[Union[str, CWLObjectType]], initialWorkdir["listing"],
+                MutableSequence[Union[str, CWLObjectType]],
+                initialWorkdir["listing"],
             ):
                 if isinstance(t, Mapping) and "entry" in t:
                     # Dirent
@@ -672,7 +673,9 @@ class CommandLineTool(Process):
                             del d["dirname"]
 
                     visit_class(
-                        entry["listing"], ("File", "Directory"), remove_dirname,
+                        entry["listing"],
+                        ("File", "Directory"),
+                        remove_dirname,
                     )
 
             visit_class(
